#ifndef __INPUT_SNESAPU___
#define __INPUT_SNESAPU___

#include "Types.h"
#include "SNESAPU.h"
#include "foobar2000.h"

#pragma pack(push, 1)
struct SPCTEXT
{
	s8	date[11];							// date dumped
	s8	songlen[3];							// song length (seconds before fading)
	s8	fadelen[5];							// fade length (milliseconds)
	s8	artist[32];							// artist
	u8	chandis;							// channels disabled
	u8	emulator;							// emulator used for dump
	u8	__r2[45];
};
struct SPCBIN
{
	u8	date;
	u8	month;
	u16	year;
	s8	__r2[7];
	u16	songlen;
	s8	__r3;
	u8	fadelen[3];
	s8	__r4;
	s8	artist[32];
	s8	__r5;
	u8	emulator;
	s8	__r6[46];
};
struct SPCHEAD
{
	s8	tag[33];							// file tag
	s8	term[2];							// tag terminator
	u8	tagkind;								// tag kind
	s8	ver;								// version #/100
	u8	pc[2];								// spc registers
	u8	a;
	u8	x;
	u8	y;
	u8	psw;
	u8	sp;
	u16	__r1;
	s8	song[32];							// song title
	s8	game[32];							// game title
	s8	dumper[16];							// name of dumper
	s8	comment[32];						// comments
	union {
		SPCBIN bin;
		SPCTEXT text;
	};
};
#pragma pack(pop)

class input_snesapu
{
  private:

	static const int buffer_duration = 20;	// larger values cause seeking distortion

	static SAPUFunc	sapu;					// SNESAPU instance
	static HINSTANCE	hDLL;				// SNESAPU dll handle
	static input_snesapu *pDLL_use;
	static critical_section m_cs;

	u32			spcfilelen;					// in-memory copy of SPC file
	u8			*spcfile;

	pfc::array_t<t_uint8> m_decode_buffer;

	u32			sapu_ticks;					// keep track of how many ms have been

	u32			sapu_samplerate;			// internal copies of cfg_* vars
	u32			sapu_bitspersample;
	u32			sapu_channels;
	u32			sapu_mixing;
	u32			sapu_interpolation;
	u32			sapu_options;
	u32			sapu_fastseek;
	u32			sapu_forcelength;
	u32			sapu_usespclength;
	u32			sapu_songlength;
	u32			sapu_fadelength;

	u32			stored_songlength;			// song length info read from spc file
	u32			stored_fadelength;
	service_ptr_t<file> m_file;
	pfc::string8 m_path;

	void try700_(const char *p_path);
	void spc_header_read_pre(SPCHEAD *spchead);
	bool spc_header_read(SPCHEAD *spchead, file_info &p_info);

  public:

	enum SNESAPU_CORE {
		CORE_APLHA2,
		CORE_41568K,
		CORE_SSDLabo
	};

	input_snesapu()
		: spcfile(NULL), stored_songlength(0), stored_fadelength(0)
	  {}
	~input_snesapu();

	static bool load_dll();
	static bool free_dll();
	static SNESAPU_CORE get_snes_core();

	bool get_dll();
	void release_dll();

	static bool g_is_our_path(const char *p_path, const char *p_extension) {
		return (::stricmp_utf8(p_extension,"spc") == 0);
	}

	int spc_header_num_test(char *str, int len);
	void get_info(file_info &p_info, abort_callback &p_abort);
	void open(service_ptr_t<file> p_filehint, const char * p_path, t_input_open_reason p_reason, abort_callback &p_abort);
	void decode_initialize(unsigned int p_flags, abort_callback &p_abort);
	bool decode_run(audio_chunk &p_chunk, abort_callback &p_abort);
	void decode_seek(double p_seconds, abort_callback &p_abort);
	bool decode_can_seek();

	t_filestats get_file_stats(abort_callback & p_abort) {return m_file->get_stats(p_abort);}
	void decode_on_idle(abort_callback & p_abort) {m_file->on_idle(p_abort);}
	bool decode_get_dynamic_info(file_info &p_out, double &p_timestamp_delta) {return false;}
	bool decode_get_dynamic_info_track(file_info &p_out, double &p_timestamp_delta) {return false;}
	static bool g_is_our_content_type(const char *p_content_type) {return false;}
	void retag(const file_info &p_info, abort_callback &p_abort) {throw exception_io_unsupported_format();}
};

#endif
